<!-- ***************************************************************************  -->
<!-- *                                                                         *  -->
<!-- * Copyright 2008-2014 Avago Technologies.  All rights reserved.           *  -->
<!-- *                                                                         *  -->
<!-- ***************************************************************************  -->
<!--
***********************************************************************************************************************
* File:              NVDATA.XSL                                                                                       *
*                                                                                                                     *
* Format:            XML Stylesheet Transformation (XSLT), version 1.0 (http://www.w3.org/TR/xslt)                    *
*                    XML Path Language (XPath), version 1.0 (http://www.w3.org/TR/xpath)                              *
*                                                                                                                     *
* Inputs:            A "peppered" & validated SAS2 NVDATA XML file.  "Peppered" means the elements in the .XML file   *
*                    include all fixed attributes which are normally hidden, such as the PageType attribute in        *
*                    ManPage0).                                                                                       *
*                                                                                                                     *
* Outputs:           An intermediate XML file which contains the configuraton page tree structure and all of the      *
*                    configuration page contents from the input .XML file re-formatted into binary.  Strings are      *
*                    converted into their binary equivalents (i.e., "Hello World" becomes 48656C6C6F20576F726C64),    *
*                    and U8/U16/U32 values are zero-padded and byte-flipped as necessary.                             *
*                                                                                                                     *
*                    Furthermore, the output XML file contains processing instruction nodes for fields whose contents *
*                    cannot be calculated or derived by this XSLT (such as offset and checksum values).  This output  *
*                    XML file should be fed through a separate program that calculates these values and converts the  *
*                    text into a pure binary format.                                                                  *
*                                                                                                                     *
* Revision History:  +=========+==========+=============+==========================================================+  *
*                    |   XSLT  | Revision |  SAS2Parser | Feature Set Description                                  |  *
*                    | Version |   Date   |   Version   |                                                          |  *
*                    +=========+==========+=============+==========================================================+  *
*                    |    2.6  | 03/13/12 | 02.00.00.01 | Add MaxPCIRevisionID in CFG_PROD_ID                      |  *
*                    |    2.5  | 08/13/09 | 02.00.00.01 | U64 swapping up and lower U32 halves incorrectly         |  *
*                    |    2.4  | 07/01/09 | 02.00.00.00 | U64 byte-flipping munges the lower U32                   |  *
*                    |    2.3  | 04/17/09 | 00.12.00.00 | <?Offset> PI is generated instead of <?DwordOffset> when |  *
*                    |         |          |             |  using sas2parser v00.13.00.00                           |  *
*                    |    2.2  | 03/23/09 | 00.12.00.00 | <?Checksum> PI was added to <ManPage16> inappropriately  |  *
*                    |    2.1  | 03/10/09 | 00.13.00.00 | Added support for <MRNvdata> root element                |  *
*                    |    2.0  | 02/23/09 | 00.12.00.00 | Added SAS2Parser-to-XSLT versioning                      |  *
*                    |    1.0  | 01/06/09 | 00.12.00.00 | Initial development                                      |  *
*                    +=========+==========+=============+==========================================================+  *
*                                                                                                                     *
***********************************************************************************************************************
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="xml" media-type="text/xml" encoding="us-ascii"/>
  <xsl:param name="GenerateExtendedHeader">Unknown</xsl:param> <!-- By default, generate the MPI_EXT_FW_HDR -->
  <xsl:param name="SAS2ParserVersion">00000000</xsl:param> <!-- The version of sas2parser being used -->
  <xsl:variable name="StylesheetVersion">2.6</xsl:variable>
  <xsl:variable name="DesiredSAS2ParserVersion">000D0000</xsl:variable> <!-- 00.13.00.00 -->
  <xsl:variable name="RESERVED_U8">00</xsl:variable>
  <xsl:variable name="RESERVED_U16">0000</xsl:variable>
  <xsl:variable name="RESERVED_U32">00000000</xsl:variable>
  <xsl:variable name="NO_FORCE_UPDATE">FF</xsl:variable>
  <xsl:variable name="FORCE_UPDATE">FE</xsl:variable>
  <xsl:variable name="CFGI_PERSISTENT_HEADER_STATE_UPDATE_COMPLETE">FC</xsl:variable>
  <xsl:variable name="CFGI_DIR_HEADER_SIGNATURE">4E69636B</xsl:variable>
  <xsl:variable name="CFGI_DIR_HEADER_STATE_VALID">F8</xsl:variable>
  <xsl:variable name="CFGI_PROD_ID_SIGNATURE">4672617A</xsl:variable>
  <xsl:variable name="CFGI_DIR_ENTRY_STATE_IN_USE">0C</xsl:variable>
  <xsl:variable name="CFGI_TERMINAL_LINK">FFFFFFFF</xsl:variable>
  <xsl:variable name="SIZEOF_CFGI_DIR_HEADER">36</xsl:variable> <!-- sizeof(CFGI_DIR_HEADER) = 36 bytes -->
  <xsl:variable name="SIZEOF_CFGI_PROD_ID">64</xsl:variable> <!-- sizeof(CFGI_PROD_ID) = 64 bytes -->
  <xsl:variable name="SIZEOF_MPI2_EXT_IMAGE_HEADER">64</xsl:variable> <!-- sizeof(MPI2_EXT_IMAGE_HEADER) = 64 bytes -->
  <xsl:variable name="MPI2_EXT_IMAGE_TYPE_NVDATA">03</xsl:variable>
  <!--
  *******************************************************************************************************
  * Section #1: "Utility" templates used for zero padding, byte-swapping, ASCII-to-hex conversion, etc. *
  *******************************************************************************************************
  -->
  <!--
  =======================================================================================================
  The VerifyVersion template checks that the version of sas2parser being used is compatible with this stylesheet.
  -->
  <xsl:template name="VerifyVersion">
    <xsl:param name="RootElementName"/>
    <xsl:param name="ActualVersion"/>
    <xsl:param name="DesiredVersion"/>
    <!-- Indicate the version of the stylesheet -->
    <xsl:message terminate="no">NVDATA XSLT Version: <xsl:value-of select="$StylesheetVersion"/></xsl:message>
    <xsl:if test="string-length($ActualVersion) = 8 and string-length($DesiredVersion) = 8">
      <xsl:variable name="ActualVersionValue">
        <xsl:call-template name="HexToDecimal">
          <xsl:with-param name="InNumber" select="substring($ActualVersion, 1, 4)"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="DesiredVersionValue">
        <xsl:call-template name="HexToDecimal">
          <xsl:with-param name="InNumber" select="substring($DesiredVersion, 1, 4)"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="$ActualVersionValue &lt; $DesiredVersionValue">
          <!-- <MRNvdata> requires version 00.13.00.00 -->
          <xsl:if test="$RootElementName = 'MRNvdata'">
            <xsl:message terminate="yes">Error: You must use SAS2Parser version <xsl:call-template name="VersionHexToString"><xsl:with-param name="InDword" select="$DesiredVersion"/></xsl:call-template> or greater to process an XML document with root element "<xsl:value-of select="$RootElementName"/>"</xsl:message>
          </xsl:if>
          <xsl:message terminate="no">Warning: This XSLT file contains features that are compatible with SAS2Parser version <xsl:call-template name="VersionHexToString"><xsl:with-param name="InDword" select="$DesiredVersion"/></xsl:call-template> or greater.</xsl:message>
          <xsl:value-of select="'false'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'true'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>
  <!--
  =======================================================================================================
  The VersionHexToString template converts an MPI Version DWORD (0x010203DB) into a string ("01.02.03.219")
  -->
  <xsl:template name="VersionHexToString">
    <xsl:param name="InDword"/>
    <xsl:if test="string-length($InDword) = 8">
      <xsl:variable name="Major">
        <xsl:call-template name="LeadingZeroPad">
          <xsl:with-param name="InString">
            <xsl:call-template name="HexToDecimal">
              <xsl:with-param name="InNumber" select="substring($InDword, 1, 2)"/>
            </xsl:call-template>
          </xsl:with-param>
          <xsl:with-param name="Length">2</xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="Minor">
        <xsl:call-template name="LeadingZeroPad">
          <xsl:with-param name="InString">
            <xsl:call-template name="HexToDecimal">
              <xsl:with-param name="InNumber" select="substring($InDword, 3, 2)"/>
            </xsl:call-template>
          </xsl:with-param>
          <xsl:with-param name="Length">2</xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="Unit">
        <xsl:call-template name="LeadingZeroPad">
          <xsl:with-param name="InString">
            <xsl:call-template name="HexToDecimal">
              <xsl:with-param name="InNumber" select="substring($InDword, 5, 2)"/>
            </xsl:call-template>
          </xsl:with-param>
          <xsl:with-param name="Length">2</xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="Dev">
        <xsl:call-template name="LeadingZeroPad">
          <xsl:with-param name="InString">
            <xsl:call-template name="HexToDecimal">
              <xsl:with-param name="InNumber" select="substring($InDword, 7, 2)"/>
            </xsl:call-template>
          </xsl:with-param>
          <xsl:with-param name="Length">2</xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="concat($Major, '.', $Minor, '.', $Unit, '.', $Dev)"/>
    </xsl:if>
  </xsl:template>
  <!--
  =======================================================================================================
  The FixedStringToHex template pads a string with trailing spaces (up to a specified length) and performs
  ASCII-to-hexadecimal conversion for the entire padded string.
  -->
  <xsl:template name="FixedStringToHex">
    <xsl:param name="InString"/>
    <xsl:param name="Length"/>
    <xsl:choose>
      <xsl:when test="string-length($InString) &gt; $Length">
        <xsl:call-template name="FixedStringToHex">
          <xsl:with-param name="InString" select="substring( $InString, 1, $Length )"/>
          <xsl:with-param name="Length" select="$Length"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <!-- Print the whole string -->
        <xsl:call-template name="StringToHex">
          <xsl:with-param name="InString" select="$InString"/>
        </xsl:call-template>
        <!-- Print training "00", if necessary -->
        <xsl:if test="string-length($InString) &lt; $Length">
          <xsl:call-template name="RepeatSequence">
            <xsl:with-param name="Sequence">00</xsl:with-param>
            <xsl:with-param name="NumInstances" select="$Length - string-length($InString)"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
  =======================================================================================================
  The StringToHex template performs ASCII-to-hexadecimal conversion for an entire string.
  -->
  <xsl:template name="StringToHex">
    <xsl:param name="InString"/>
    <xsl:call-template name="CharToHex">
      <xsl:with-param name="InChar" select="substring($InString, 1, 1)"/>
    </xsl:call-template>
    <xsl:if test="string-length($InString) > 1">
      <xsl:call-template name="StringToHex">
        <xsl:with-param name="InString" select="substring($InString, 2)"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <!--
  =======================================================================================================
  The CharToHex template performs SINGLE CHARACTER ASCII-to-hexadecimal conversion.
  -->
  <xsl:template name="CharToHex">
    <xsl:param name="InChar"></xsl:param>
    <xsl:variable name="Characters"> !"#$%&amp;'()*+,-./0123456789:;&lt;=&gt;?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz~</xsl:variable>
    <xsl:if test="boolean(string-length($InChar) = 1 and contains($Characters, $InChar))">
      <xsl:variable name="PostCharacters" select="substring-after($Characters, $InChar)"/>
      <xsl:variable name="ASCIIValue" select="number(string-length($Characters) - string-length($PostCharacters) + 32 - 1)"/>
      <xsl:call-template name="DecimalToHex">
        <xsl:with-param name="InNumber" select="$ASCIIValue"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <!--
  =======================================================================================================
  The DecimalToHex template performs base 10 to base 16 number conversion for a number < 255.
  -->
  <xsl:template name="DecimalToHex">
    <xsl:param name="InNumber"/>
    <xsl:if test="number($InNumber) &lt;= 255">
      <xsl:variable name="ZeroToF">0123456789ABCDEF</xsl:variable>
      <xsl:value-of select="substring($ZeroToF, floor(number($InNumber) div 16) + 1, 1)"/>
      <xsl:value-of select="substring($ZeroToF, number(number($InNumber) mod 16) + 1, 1)"/>
    </xsl:if>
  </xsl:template>
  <!--
  =======================================================================================================
    The Exp template performs the exponentiation function (i.e. n^m or n raised to the m power)
  -->
  <xsl:template name="Exp">
      <xsl:param name="Base"/>
    <xsl:param name="Exponent"/>
    <xsl:choose>
      <xsl:when test="$Exponent = 0">
        <xsl:value-of select="1"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="Lower">
          <xsl:call-template name="Exp">
            <xsl:with-param name="Base" select="number($Base)"/>
            <xsl:with-param name="Exponent" select="number($Exponent - 1)"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="number($Base * $Lower)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
  =======================================================================================================
  The HexToDecimal template performs base 16 to base 10 number conversion (no length limitation)
    An arbitrary hex value 0xKLMN is really (K * 16^3) + (L * 16^2) + (M * 16^1) + (N * 16^0)
  -->
  <xsl:template name="HexToDecimal">
      <xsl:param name="InNumber"/>
      <xsl:variable name="FToZero">FEDCBA9876543210</xsl:variable>
      <xsl:choose>
      <xsl:when test="string-length($InNumber) = 1">
        <xsl:value-of select="string-length(substring-after($FToZero, $InNumber))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="Multiplicand" select="string-length(substring-after($FToZero, substring($InNumber, 1, 1)))"/>
        <xsl:variable name="Multiplier">
          <xsl:call-template name="Exp">
            <xsl:with-param name="Base">16</xsl:with-param>
            <xsl:with-param name="Exponent" select="number(string-length($InNumber) - 1)"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="ThisDigit" select="number($Multiplicand * $Multiplier)"/>
        <xsl:variable name="NextDigit">
          <xsl:call-template name="HexToDecimal">
            <xsl:with-param name="InNumber" select="substring($InNumber, 2)"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="number($ThisDigit + $NextDigit)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
  =======================================================================================================
  The U8 template takes the Value parameter and pads it with leading zeroes so the length is 2.
  -->
  <xsl:template name="U8">
    <xsl:param name="Value"/>
    <xsl:call-template name="LeadingZeroPad">
      <xsl:with-param name="InString" select="$Value"/>
      <xsl:with-param name="Length">2</xsl:with-param>
    </xsl:call-template>
  </xsl:template>
  <!--
  =======================================================================================================
  The U16 template takes the Value parameter, pads it with leading zeroes so the length is 4, and then
  performs U16 byte-swapping.
  -->
  <xsl:template name="U16">
    <xsl:param name="Value"/>
    <xsl:variable name="PaddedString">
      <xsl:call-template name="LeadingZeroPad">
        <xsl:with-param name="InString" select="$Value"/>
        <xsl:with-param name="Length">4</xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="concat(substring($PaddedString, 3, 2), substring($PaddedString, 1, 2))"/>
  </xsl:template>
  <!--
  =======================================================================================================
  The U32 template takes the Value parameter, pads it with leading zeroes so the length is 8, and then
  performs U32 byte-swapping.
  -->
  <xsl:template name="U32">
    <xsl:param name="Value"/>
    <xsl:variable name="PaddedString">
      <xsl:call-template name="LeadingZeroPad">
        <xsl:with-param name="InString" select="$Value"/>
        <xsl:with-param name="Length">8</xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="concat(substring($PaddedString, 7, 2), substring($PaddedString, 5, 2), substring($PaddedString, 3, 2), substring($PaddedString, 1, 2))"/>
  </xsl:template>
  <!--
  =======================================================================================================
  The U64 template takes the Value parameter, pads it with leading zeroes so the length is 16, and then
  performs U64 byte-swapping (which is U32 byte swapping on the upper and lower halves of the U64).
  -->
  <xsl:template name="U64">
    <xsl:param name="Value"/>
    <xsl:variable name="PaddedString">
      <xsl:call-template name="LeadingZeroPad">
        <xsl:with-param name="InString" select="$Value"/>
        <xsl:with-param name="Length">16</xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    <!-- Byte-flip the high and low as two U32s -->
    <xsl:value-of select="concat(substring($PaddedString, 15, 2), substring($PaddedString, 13, 2), substring($PaddedString, 11, 2), substring($PaddedString, 9, 2))"/>
    <xsl:value-of select="concat(substring($PaddedString, 7, 2), substring($PaddedString, 5, 2), substring($PaddedString, 3, 2), substring($PaddedString, 1, 2))"/>
  </xsl:template>
  <!--
  =======================================================================================================
  The RepeatSequence template outputs a sequence of characters a specified number of times.
  -->
  <xsl:template name="RepeatSequence">
    <xsl:param name="Sequence"/>
    <xsl:param name="NumInstances"/>
    <xsl:if test="number($NumInstances) > 0">
      <xsl:value-of select="$Sequence"/>
      <xsl:call-template name="RepeatSequence">
        <xsl:with-param name="Sequence" select="$Sequence"/>
        <xsl:with-param name="NumInstances" select="number($NumInstances) - 1"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <!--
  =======================================================================================================
  The LeadingZeroPad template pads the InString parameter with the specified number of leading zeroes.
  -->
  <xsl:template name="LeadingZeroPad">
    <xsl:param name="InString"/>
    <xsl:param name="Length"/>
    <xsl:choose>
      <xsl:when test="string-length($InString) &lt; $Length">
        <xsl:call-template name="LeadingZeroPad">
          <xsl:with-param name="InString" select="concat( '0', $InString )"/>
          <xsl:with-param name="Length" select="$Length"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="string-length($InString) &gt; $Length">
        <xsl:call-template name="LeadingZeroPad">
          <xsl:with-param name="InString" select="substring( $InString, string-length($InString) - $Length )"/>
          <xsl:with-param name="Length" select="$Length"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$InString"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!--
  =======================================================================================================
  The DebugInformation template prints node and attribute information and is not used except for
  debugging purposes.
  -->
  <xsl:template name="DebugInformation">
    <xsl:element name="DebugInformation">
      <xsl:element name="NodeName">
        <xsl:value-of select="name(.)"/>
      </xsl:element>
      <xsl:element name="AllAttributes">
        <xsl:for-each select="./@*">
          <xsl:element name="{name(.)}">
            <xsl:value-of select="."/>
          </xsl:element>
        </xsl:for-each>
      </xsl:element>
    </xsl:element>
  </xsl:template>
  <!--
  ********************************************************************************************************
  * Section #2: "Helper" templates to generate NVDATA config directory structures & process config data. *
  ********************************************************************************************************
  -->
  <xsl:template name="MPI2_EXT_IMAGE_HEADER">
    <xsl:element name="MPI2_EXT_IMAGE_HEADER">
      <!--
          #define MPI2_EXT_IMAGE_TYPE_NVDATA              (0x03)

          /* Extended Image Header */
          typedef struct _MPI2_EXT_IMAGE_HEADER
          {
              U8                      ImageType;                  /* 0x00 */
              U8                      Reserved1;                  /* 0x01 */
              U16                     Reserved2;                  /* 0x02 */
              U32                     Checksum;                   /* 0x04 */
              U32                     ImageSize;                  /* 0x08 */ size of NVDATA + size of header
              U32                     NextImageHeaderOffset;      /* 0x0C */ 00000000
              U32                     PackageVersion;             /* 0x10 */ 00000000
              U32                     Reserved3;                  /* 0x14 */
              U32                     Reserved4;                  /* 0x18 */
              U32                     Reserved5;                  /* 0x1C */
              U8                      IdentifyString[32];         /* 0x20 */
          } MPI2_EXT_IMAGE_HEADER, MPI2_POINTER PTR_MPI2_EXT_IMAGE_HEADER,
            Mpi2ExtImageHeader_t, MPI2_POINTER pMpi2ExtImageHeader_t;
      -->
      <xsl:element name="ImageType">
        <xsl:value-of select="$MPI2_EXT_IMAGE_TYPE_NVDATA"/>
      </xsl:element>
      <xsl:element name="Reserved1">
        <xsl:value-of select="$RESERVED_U8"/>
      </xsl:element>
      <xsl:element name="Reserved2">
        <xsl:value-of select="$RESERVED_U16"/>
      </xsl:element>
      <xsl:element name="Checksum">
        <xsl:processing-instruction name="Checksum">Type="U32" Algorithm="ExtendedImage" Element="ExtendedImage"</xsl:processing-instruction>
      </xsl:element>
      <xsl:element name="ImageSize">
        <xsl:processing-instruction name="BytesWritten">Type="U32" Element="ExtendedImage" AdjustmentBytes="<xsl:call-template name="DecimalToHex"><xsl:with-param name="InNumber" select="$SIZEOF_MPI2_EXT_IMAGE_HEADER"/></xsl:call-template>"</xsl:processing-instruction>
      </xsl:element>
      <xsl:element name="NextImageHeaderOffset">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="PackageVersion">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="Reserved3">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="Reserved4">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="Reserved5">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="IdentifyString">
        <xsl:call-template name="RepeatSequence">
            <xsl:with-param name="Sequence">00</xsl:with-param>
            <xsl:with-param name="NumInstances">32</xsl:with-param>
        </xsl:call-template>
      </xsl:element>
    </xsl:element>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template match="GeneralData">
    <xsl:param name="CDH_SIZE" />
    <xsl:param name="CDE_SIZE" />
    <xsl:param name="PPH_SIZE" />
    <xsl:param name="PROD_ID_SIZE" />
    <xsl:element name="CFGI_DIR_HEADER">
      <!--
        #define CFGI_DIR_HEADER_SIGNATURE              (0x4E69636B)

        #define CFGI_DIR_HEADER_STATE_ERASED           (0xFF)
        #define CFGI_DIR_HEADER_STATE_INITIALIZATION   (CFGI_DIR_HEADER_STATE_ERASED            & ~0x01)  /* 0xFE */
        #define CFGI_DIR_HEADER_STATE_RCV_DATA         (CFGI_DIR_HEADER_STATE_INITIALIZATION    & ~0x02)  /* 0xFC */
        #define CFGI_DIR_HEADER_STATE_VALID            (CFGI_DIR_HEADER_STATE_RCV_DATA          & ~0x04)  /* 0xF8 */
        #define CFGI_DIR_HEADER_STATE_XFER_COMPLETE    (CFGI_DIR_HEADER_STATE_VALID             & ~0x08)  /* 0xF0 */

        typedef struct _CFGI_DIR_HEADER
        {
            U32         Signature;                  /* 0x00 */
            U8          State;                      /* 0x04 */
            U8          Reserved1;                  /* 0x05 Reserved for Checksum */
            U16         TotalBytes;                 /* 0x06 Seeprom only */
            U32         NvdataVersion;              /* 0x08 NVDATA version  */
            U8          CdhSize;                    /* 0x0C CFGI_DIR_HEADER size in DWORDS */
            U8          CdeSize;                    /* 0x0D CFGI_DIR_ENTRY size in DWORDS */
            U8          PphSize;                    /* 0x0E CFGI_PERSISTENT_PAGE_HEADER size in DWORDS */
            U8          ProdIdSize;                 /* 0x0F CFGI_PROD_ID size in DWORDS */
            U32         NbrDirEntries;              /* 0x10 # of Dir Entries */
            U32         NbrPersistDirEntries;       /* 0x14 # of Dir Entries that have PERSISTENT Attribute */
            U32         Reserved3;                  /* 0x18 */
            U16         ProductIdOffset;            /* 0x1C Offset to CFGI_PROD_ID in DWORDS */
            U16         DirEntryOffset;             /* 0x1E Offset to CFGI_DIR_ENTRY in DWORDS */
            U32         VendorNvdataVersion;        /* 0x20 Vendor's NVDATA Version */
        } CFGI_DIR_HEADER, *PTR_CFGI_DIR_HEADER;
      -->
      <xsl:element name="Signature">
        <xsl:call-template name="U32">
          <xsl:with-param name="Value" select="$CFGI_DIR_HEADER_SIGNATURE"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="State">
        <xsl:value-of select="$CFGI_DIR_HEADER_STATE_VALID" />
      </xsl:element>
      <xsl:element name="Reserved1">
        <xsl:value-of select="$RESERVED_U8" />
      </xsl:element>
      <xsl:element name="TotalBytes">
        <xsl:processing-instruction name="BytesWritten">Type="U16" Element="ConfigDirectory" AdjustmentBytes="<xsl:call-template name="DecimalToHex"><xsl:with-param name="InNumber" select="number($SIZEOF_CFGI_DIR_HEADER + $SIZEOF_CFGI_PROD_ID)"/></xsl:call-template>"</xsl:processing-instruction>
      </xsl:element>
      <xsl:element name="NvdataVersion">
        <xsl:choose>
          <xsl:when test="current()/NvdataVersion">
            <xsl:call-template name="U32">
              <xsl:with-param name="Value" select="current()/NvdataVersion" />
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="current()/NvdataMajorVersion and current()/NvdataMinorVersion">
              <xsl:call-template name="U16">
                <xsl:with-param name="Value" select="current()/NvdataMinorVersion"/>
              </xsl:call-template>
              <xsl:call-template name="U16">
                <xsl:with-param name="Value" select="current()/NvdataMajorVersion"/>
              </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:message terminate="yes">XSLT Error: Could not identify NVDATA Version!</xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
      <xsl:element name="CdhSize">
        <xsl:value-of select="$CDH_SIZE" />
      </xsl:element>
      <xsl:element name="CdeSize">
        <xsl:value-of select="$CDE_SIZE" />
      </xsl:element>
      <xsl:element name="PphSize">
        <xsl:value-of select="$PPH_SIZE" />
      </xsl:element>
      <xsl:element name="ProdIdSize">
        <xsl:value-of select="$PROD_ID_SIZE" />
      </xsl:element>
      <xsl:element name="NbrDirEntries">
        <xsl:call-template name="U32">
          <!-- The current context node is SAS2Nvdata/Adapter/GeneralData.'
               Go up one node to SAS2Nvdata/Adapter, and count all of the children (except GeneralData) -->
          <xsl:with-param name="Value">
            <xsl:call-template name="DecimalToHex">
              <xsl:with-param name="InNumber" select="count(../*) - 1"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="NbrPersistDirEntries">
        <xsl:call-template name="U32">
          <!-- The current context node is SAS2Nvdata/Adapter/GeneralData.'
               Go up one node to SAS2Nvdata/Adapter, and count all of the children (except GeneralData) -->
          <xsl:with-param name="Value">
            <xsl:call-template name="DecimalToHex">
              <xsl:with-param name="InNumber" select="count(../*) - 1"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="Reserved3">
        <xsl:value-of select="$RESERVED_U32" />
      </xsl:element>
      <xsl:element name="ProdIdOffset">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value">
            <xsl:call-template name="DecimalToHex">
              <xsl:with-param name="InNumber" select="number($SIZEOF_CFGI_DIR_HEADER div 4)"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="DirEntryOffset">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value">
            <xsl:call-template name="DecimalToHex">
              <xsl:with-param name="InNumber" select="number($SIZEOF_CFGI_DIR_HEADER div 4) + number($SIZEOF_CFGI_PROD_ID div 4)"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="VendorNvdataVersion">
        <xsl:value-of select="$RESERVED_U32" />
      </xsl:element>
    </xsl:element>
    <!-- end-of-GeneralData -->
    <!-- start-of-ProductId -->
    <xsl:element name="CFGI_PROD_ID">
      <!--
        #define CFGI_PROD_ID_SIGNATURE              (0x4672617A)

        typedef struct _CFGI_PROD_ID
        {
            U32 Signature;
            U8  VendorId[8];          /* ASCII Vendor ID */
            U8  ProductId[16];        /* ASCII Product ID */
            U8  ProductRevision[4];   /* Product Revision */
            U32 Reserved1;
            U32 Reserved2;
            U16 PCIVendorID;          /* indicates the exact NVDATA image for which it is intended */
                                      /* 0xFFFF = matches any combination of PCI Vendor ID, PCI Device ID, */
                                      /*          PCI Subsystem Vendor ID, and PCI Subsystem ID            */
            U16 PCIDeviceID;          /* Indicates the exact NVDATA image for which this image is intended */
            U16 PCISubsystemVendorID; /* Indicates the exact NVDATA image for which this image is intended */
            U16 PCISubsystemID;       /* Indicates the exact NVDATA image for which this image is intended */
            U8  PCIRevisionID;        /* Indicates the exact NVDATA image for which this image is intended */
                                      /* 0xFF, indicates this image may be used with any of the chips indicated by the four fields above */
            U8  SubOEMID;             /* Application specified 8-bit binary field */
            U8  MaxPCIRevisionID;     /* Max supported PCI Revision for backward compatibility */
            U8  Reserved5;
            U32 Reserved6;
            U32 Reserved7;
            U32 Reserved8;
        } CFGI_PROD_ID, *PTR_CFGI_PROD_ID;
      -->
      <xsl:element name="Signature">
        <xsl:call-template name="U32">
          <xsl:with-param name="Value" select="$CFGI_PROD_ID_SIGNATURE"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="VendorId">
        <xsl:call-template name="FixedStringToHex">
          <xsl:with-param name="InString" select="current()/NvdataVendorId"/>
          <xsl:with-param name="Length">8</xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="ProductId">
        <xsl:call-template name="FixedStringToHex">
          <xsl:with-param name="InString" select="current()/NvdataProductId"/>
          <xsl:with-param name="Length">16</xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="ProductRevision">
        <xsl:call-template name="FixedStringToHex">
          <xsl:with-param name="InString" select="current()/NvdataProductRevision"/>
          <xsl:with-param name="Length">4</xsl:with-param>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="Reserved1">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="Reserved2">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="PCIVendorID">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value" select="current()/PCIVendorID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="PCIDeviceID">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value" select="current()/PCIDeviceID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="PCISubsystemVendorID">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value" select="current()/PCISubsystemVendorID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="PCISubsystemID">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value" select="current()/PCISubsystemID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="PCIRevisionID">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="current()/PCIRevisionID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="SubOEMID">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="current()/SubOEMID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="MaxPCIRevisionID">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="current()/MaxPCIRevisionID"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="Reserved5">
        <xsl:value-of select="$RESERVED_U8"/>
      </xsl:element>
      <xsl:element name="Reserved6">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="Reserved7">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
      <xsl:element name="Reserved8">
        <xsl:value-of select="$RESERVED_U32"/>
      </xsl:element>
    </xsl:element>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template name="CFGI_DIR_ENTRY">
    <xsl:param name="IsVersionCompatible"/>
    <xsl:param name="NodeName"></xsl:param>
    <xsl:element name="CFGI_DIR_ENTRY">
      <!--
        #define CFGI_DIR_ENTRY_STATE_IN_USE           (CFGI_DIR_ENTRY_STATE_BEGIN_UPDATE & ~0x2)  /* 0xC */

        typedef struct _CFGI_DIR_ENTRY
        {
            U8  State;
            U8  Reserved;
            U16 AllocUnits;         /* Number of DWORDs */
            U8  PageType;
            U8  PageNum;
            U8  UpdateFlags;
            U8  IocNum;
            U32 DwordOffset;
            U32 PageAddress;        /* Instance number of this page type */
        } CFGI_DIR_ENTRY, *PTR_CFGI_DIR_ENTRY;
        -->
      <xsl:element name="State">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="$CFGI_DIR_ENTRY_STATE_IN_USE"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="Reserved">
        <xsl:value-of select="$RESERVED_U8"/>
      </xsl:element>
      <xsl:element name="AllocUnits">
        <xsl:call-template name="U16">
          <xsl:with-param name="Value" select="current()/@PageLength"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="PageType">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="current()/@PageType"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="PageNum">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="current()/@PageNumber"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="UpdateFlags">
        <xsl:if test="current()/@ForceUpdate = string(true())">
          <xsl:value-of select="$FORCE_UPDATE"/>
        </xsl:if>
        <xsl:if test="current()/@ForceUpdate = string(false())">
          <xsl:value-of select="$NO_FORCE_UPDATE"/>
        </xsl:if>
      </xsl:element>
      <xsl:element name="IocNum">
        <xsl:call-template name="U8">
          <xsl:with-param name="Value" select="current()/@IocNum"/>
        </xsl:call-template>
      </xsl:element>
      <xsl:element name="DwordOffset">
        <xsl:choose>
          <xsl:when test="$IsVersionCompatible = 'true'">
            <xsl:choose>
              <xsl:when test="$GenerateExtendedHeader = 'true'">
                <xsl:processing-instruction name="Offset">Element="<xsl:value-of select="$NodeName"/>" Type="U32" AdjustmentBytes="-<xsl:call-template name="DecimalToHex"><xsl:with-param name="InNumber" select="$SIZEOF_MPI2_EXT_IMAGE_HEADER"/></xsl:call-template>"  Units="U32"</xsl:processing-instruction>
              </xsl:when>
              <xsl:otherwise>
                <xsl:processing-instruction name="Offset">Element="<xsl:value-of select="$NodeName"/>" Type="U32" Units="U32"</xsl:processing-instruction>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:choose>
          <xsl:when test="$GenerateExtendedHeader = 'true'">
            <xsl:processing-instruction name="DwordOffset">Element="<xsl:value-of select="$NodeName"/>" Type="U32" AdjustmentBytes="-<xsl:call-template name="DecimalToHex"><xsl:with-param name="InNumber" select="$SIZEOF_MPI2_EXT_IMAGE_HEADER"/></xsl:call-template>"</xsl:processing-instruction>
          </xsl:when>
          <xsl:otherwise>
            <xsl:processing-instruction name="DwordOffset">Element="<xsl:value-of select="$NodeName"/>" Type="U32"</xsl:processing-instruction>
          </xsl:otherwise>
        </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
      <xsl:element name="PageAddress">
        <xsl:call-template name="U32">
          <xsl:with-param name="Value" select="current()/@PageAddress"/>
        </xsl:call-template>
      </xsl:element>
    </xsl:element>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template name="ChecksumCalculation">
    <xsl:param name="IsVersionCompatible"/>
    <xsl:choose>
      <xsl:when test="$IsVersionCompatible = 'true'">
        <xsl:if test="@ChecksumSeed and @ChecksumStart and @ChecksumLength and @ChecksumLocation and @ChecksumAlgorithm">
          <xsl:element name="ConfigPageChecksum">
            <xsl:processing-instruction name="Checksum">Type="U8" Element="PERSISTENT_PAGE_DATA" Algorithm="<xsl:value-of select="@ChecksumAlgorithm"/>" ChecksumSeed="<xsl:value-of select="@ChecksumSeed"/>" ChecksumStart="<xsl:value-of select="@ChecksumStart"/>" ChecksumLength="<xsl:value-of select="@ChecksumLength"/>" ChecksumLocation="<xsl:value-of select="@ChecksumLocation"/>"</xsl:processing-instruction>
          </xsl:element>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="@ChecksumSeed and @ChecksumStart and @ChecksumLength and @ChecksumLocation and @ChecksumAlgorithm='SBR'">
          <xsl:element name="ConfigPageChecksum">
            <xsl:processing-instruction name="Checksum">Type="U8" Element="PERSISTENT_PAGE_DATA" Algorithm="SBR" SBRChecksumSeed="<xsl:value-of select="@ChecksumSeed"/>" SBRChecksumStart="<xsl:value-of select="@ChecksumStart"/>" SBRChecksumLength="<xsl:value-of select="@ChecksumLength"/>" SBRChecksumLocation="<xsl:value-of select="@ChecksumLocation"/>"</xsl:processing-instruction>
          </xsl:element>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template name="PERSISTENT_PAGE_HEADER">
    <xsl:element name="PERSISTENT_PAGE_HEADER">
      <!--
        #define CFGI_PERSISTENT_HEADER_STATE_ERASED          (0xFF)
        #define CFGI_PERSISTENT_HEADER_STATE_BEGIN_UPDATE    (CFGI_PERSISTENT_HEADER_STATE_ERASED       & ~0x01)  /* 0xFE */
        #define CFGI_PERSISTENT_HEADER_STATE_UPDATE_COMPLETE (CFGI_PERSISTENT_HEADER_STATE_BEGIN_UPDATE & ~0x02)  /* 0xFC */

        typedef struct _CFGI_PERSISTENT_PAGE_HEADER
        {
            U8          State;
            U8          Checksum;
            U16         Reserved;
            U32         DwordOffset;
        } CFGI_PERSISTENT_PAGE_HEADER, *PTR_CFGI_PERSISTENT_PAGE_HEADER;

        #define CFGI_TERMINAL_LINK                    (0xFFFFFFFF)    /* This indicates that there are no more entries */
      -->
      <xsl:element name="State">
        <xsl:value-of select="$CFGI_PERSISTENT_HEADER_STATE_UPDATE_COMPLETE"/>
      </xsl:element>
      <xsl:element name="Checksum">
        <xsl:processing-instruction name="Checksum">Type="U8" Algorithm="PersistentPage" Element="PERSISTENT_PAGE_DATA"</xsl:processing-instruction>
      </xsl:element>
      <xsl:element name="Reserved">
        <xsl:value-of select="$RESERVED_U16"/>
      </xsl:element>
      <xsl:element name="DwordOffset">
        <xsl:value-of select="$CFGI_TERMINAL_LINK"/>
      </xsl:element>
    </xsl:element>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template name="PERSISTENT_PAGE_DATA">
    <xsl:param name="PageName"/>
    <!-- Start-Of-Config-Page <xsl:value-of select="$PageName"/> -->
    <xsl:element name="PERSISTENT_PAGE_DATA">
      <xsl:call-template name="ConfigPageData"/>
    </xsl:element>
    <!-- End-Of-Config-Page -->
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template name="ConfigPageData">
    <xsl:for-each select="./*">
      <xsl:element name="{name(.)}">
      <!-- Perform output for current element -->
      <xsl:choose>
        <xsl:when test="count(./*) &gt; 0">
          <!--  Recursively call this routine for any child elements-->
          <xsl:call-template name="ConfigPageData"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="./@type = 'string'">
              <xsl:call-template name="FixedStringToHex">
                <xsl:with-param name="InString" select="."/>
                <xsl:with-param name="Length">
                  <xsl:call-template name="HexToDecimal">
                    <xsl:with-param name="InNumber" select="./@size"/>
                  </xsl:call-template>
                </xsl:with-param>
              </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
              <xsl:choose>
                <!-- U8 -->
                <xsl:when test="number(./@size) = 1">
                  <xsl:call-template name="U8">
                    <xsl:with-param name="Value" select="."/>
                  </xsl:call-template>
                </xsl:when>
                <!-- U16 -->
                <xsl:when test="number(./@size) = 2">
                  <xsl:call-template name="U16">
                    <xsl:with-param name="Value" select="."/>
                  </xsl:call-template>
                </xsl:when>
                <!-- U32 -->
                <xsl:when test="number(./@size) = 4">
                  <xsl:call-template name="U32">
                    <xsl:with-param name="Value" select="."/>
                  </xsl:call-template>
                </xsl:when>
                <!-- U64 -->
                <xsl:when test="number(./@size) = 8">
                  <xsl:call-template name="U64">
                    <xsl:with-param name="Value" select="."/>
                  </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:message terminate="yes">XSLT Processing Error: Unknown Data Type Encountered!</xsl:message>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
      </xsl:element>
    </xsl:for-each>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template match="Adapter">
    <xsl:param name="IsVersionCompatible"/>
    <!--
    The format of the binary
    NVDATA follows the following
    pattern:

    +=======================+
    |   Config Directory    |
    |         Header        |
    +=======================+
    |       Product ID      |
    |         Header        |
    +=======================+
    |    Config Directory   |
    |       Entry For       |
    | PageType=1, PageNum=1 |
    +=======================+
             ...
    +=======================+
    |   Config Directory    |
    |       Entry For       |
    | PageType=n, PageNum=n |
    +=======================+
    |    Persistent Page    |
    |       Header For      |
    | PageType=1, PageNum=1 |
    +=======================+
    | Persistent Page Data  |
    |         For           |
    | PageType=1, PageNum=1 |
    +=======================+
    |    Persistent Page    |
    |       Header For      |
    | PageType=1, PageNum=2 |
    +=======================+
    | Persistent Page Data  |
    |         For           |
    | PageType=1, PageNum=2 |
    +=======================+
              ...
    +=======================+
    |    Persistent Page    |
    |       Header For      |
    | PageType=n, PageNum=n |
    +=======================+
    | Persistent Page Data  |
    |         For           |
    | PageType=n, PageNum=n |
    +=======================+
    -->
    <!-- start-of-Adapter -->
    <!-- Generate the config directory header and product ID header -->
    <xsl:element name="ExtendedImage">
      <xsl:apply-templates select="GeneralData">
        <xsl:with-param name="CDH_SIZE">09</xsl:with-param>
        <xsl:with-param name="CDE_SIZE">04</xsl:with-param>
        <xsl:with-param name="PPH_SIZE">02</xsl:with-param>
        <xsl:with-param name="PROD_ID_SIZE">10</xsl:with-param>
      </xsl:apply-templates>
      <xsl:element name="ConfigDirectory">
        <!-- Generate all of the config directory entries -->
        <xsl:for-each select="*">
          <xsl:if test="name(.) != 'GeneralData'">
            <xsl:call-template name="CFGI_DIR_ENTRY">
              <xsl:with-param name="NodeName" select="name(.)"/>
              <xsl:with-param name="IsVersionCompatible" select="$IsVersionCompatible"/>
            </xsl:call-template>
          </xsl:if>
        </xsl:for-each>
        <!-- Generate each page's persistent page header + persistent page data -->
        <xsl:for-each select="*">
          <xsl:if test="name(.) != 'GeneralData'">
            <xsl:element name="{name(.)}">
              <!-- Does an element in this page require a checksum calculation? -->
              <xsl:call-template name="ChecksumCalculation">
                <xsl:with-param name="IsVersionCompatible" select="$IsVersionCompatible"/>
              </xsl:call-template>
              <!-- Calculate the checksum/length values for the header -->
              <xsl:call-template name="PERSISTENT_PAGE_HEADER">
                <xsl:with-param name="PageName" select="name(.)"/>
              </xsl:call-template>
              <!-- Process the page data -->
              <xsl:call-template name="PERSISTENT_PAGE_DATA">
                <xsl:with-param name="PageName" select="name(.)"/>
              </xsl:call-template>
            </xsl:element>
          </xsl:if>
        </xsl:for-each>
        </xsl:element>
    </xsl:element>
    <!-- end-of-Adapter -->
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template match="SAS2Nvdata">
    <!-- Check for deprecated sas2parser version -->
    <xsl:variable name="IsVersionCompatible">
      <xsl:call-template name="VerifyVersion">
        <xsl:with-param name="RootElementName">SAS2Nvdata</xsl:with-param>
        <xsl:with-param name="ActualVersion" select="$SAS2ParserVersion"/>
        <xsl:with-param name="DesiredVersion" select="$DesiredSAS2ParserVersion"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Check for missing "GenerateExtendedHeader" parameter -->
    <xsl:if test="$GenerateExtendedHeader = 'Unknown'">
      <xsl:message terminate="yes">XSLT Error: No value has been specified for the required GenerateExtendedHeader parameter.</xsl:message>
    </xsl:if>
    <xsl:element name="SAS2BinaryNvdata">
      <xsl:if test="$GenerateExtendedHeader = 'true'">
        <xsl:call-template name="MPI2_EXT_IMAGE_HEADER"></xsl:call-template>
      </xsl:if>
      <!-- start-of-NVDATA -->
      <xsl:apply-templates select="Adapter">
        <xsl:with-param name="IsVersionCompatible" select="$IsVersionCompatible"/>
      </xsl:apply-templates>
      <!-- end-of-NVDATA -->
    </xsl:element>
  </xsl:template>
  <!-- ======================================================================================================= -->
  <xsl:template match="MRNvdata">
    <!-- Check for deprecated sas2parser version -->
    <xsl:variable name="IsVersionCompatible">
      <xsl:call-template name="VerifyVersion">
        <xsl:with-param name="RootElementName">MRNvdata</xsl:with-param>
        <xsl:with-param name="ActualVersion" select="$SAS2ParserVersion"/>
        <xsl:with-param name="DesiredVersion" select="$DesiredSAS2ParserVersion"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Check for missing "GenerateExtendedHeader" parameter -->
    <xsl:if test="$GenerateExtendedHeader = 'Unknown'">
      <xsl:message terminate="yes">XSLT Error: No value has been specified for the required GenerateExtendedHeader parameter.</xsl:message>
    </xsl:if>
    <xsl:element name="MRBinaryNvdata">
      <xsl:if test="$GenerateExtendedHeader = 'true'">
        <xsl:call-template name="MPI2_EXT_IMAGE_HEADER"></xsl:call-template>
      </xsl:if>
      <!-- start-of-NVDATA -->
      <xsl:apply-templates select="Adapter">
        <xsl:with-param name="IsVersionCompatible" select="$IsVersionCompatible"/>
      </xsl:apply-templates>
      <!-- end-of-NVDATA -->
    </xsl:element>
  </xsl:template>
</xsl:stylesheet>